import torch
import numpy as np
from tqdm import tqdm
import gc

import sys
sys.path.append("")


from src.doob_h.doob_h import doob_h_tensorized

def model_based_langevin_monte_carlo(model, num_samples, num_steps, step_size, T=10, device='cuda'):
    x = torch.randn(num_samples, model.input_dim).to(device)
    model.eval()
    for i in tqdm(range(num_steps)):
        with torch.no_grad():
            t = T - T / num_steps * i
            batch_t = torch.ones((x.shape[0], 1)).to(device) * t
            score =model(torch.cat([x, batch_t], axis=1)) # model(torch.concat([x, batch_t], axis=1))
            if i < num_steps - 1:
                noise = torch.randn(num_samples, model.input_dim).to(device)
            else:
                noise = 0
            x = x + (x + 2 * score) * step_size + np.sqrt(2 * step_size) * noise
    return x

def new_sbm_based_langevin_monte_carlo(model_sbm, num_samples, num_steps, T=10, device='cuda'):
    ## parallelized version
    step_size = T / num_steps
    # input_dim = model_sbm.module.input_dim
    try:
        input_dim = model_sbm.module.input_dim if hasattr(model_sbm, 'module') else model_sbm.input_dim
    except:
        input_dim = 2
        print("input_dim is set to 2")
    x = torch.randn(num_samples, input_dim).to(device)
    model_sbm.eval()
    for i in tqdm(range(num_steps), leave=False):
        with torch.no_grad():
            t = T - T / num_steps * i
            batch_t = torch.ones((x.shape[0], 1)).to(device) * t
            score =model_sbm(torch.cat([x, batch_t], axis=1)) # model(torch.concat([x, batch_t], axis=1))
            if i < num_steps - 1:
                noise = torch.randn(num_samples, input_dim).to(device)
            else:
                noise = 0
            x = x + (x + 2 * score) * step_size + np.sqrt(2 * step_size) * noise
    return x

def doob_langevin_monte_carlo_modified(model, model_potential, num_samples, num_steps, T=10, device = "cuda"):
    with_ref = False
    step_size = T / num_steps
    input_dim = model.module.input_dim
    x = torch.randn(num_samples, input_dim).to(device)
    if with_ref:
        x_ref = x.clone()
    model.eval()
    model_potential.eval()
    for i in tqdm(range(num_steps), leave=False):
        t = T - T / num_steps * i ## t = T to 0
        batch_t = torch.ones((x.shape[0], 1)).to(device) * t
        ## x: (num_samples, input_dim)
        doob = doob_h_tensorized(model, model_potential, x, t, step_size)
        doob = torch.stack([doob[0], doob[1]], axis=1)
        
        with torch.no_grad():
            score = model(torch.cat([x, batch_t], axis=1))
            if with_ref:
                score_ref = model(torch.cat([x_ref, batch_t], axis=1))
            if i < num_steps - 1:
                noise = torch.randn(num_samples, input_dim).to(device)
            else:
                noise = 0

            x     = x     + (x     + 2 * score + 2 * doob) * step_size + np.sqrt(2 * step_size) * noise
            if with_ref:
                x_ref = x_ref + (x_ref + 2 * score_ref)        * step_size + np.sqrt(2 * step_size) * noise
            # print("doob",doob)
            # print("score",score)
        del batch_t, doob, score, noise
        if with_ref:
            del score_ref
        gc.collect()
        torch.cuda.empty_cache()

    if with_ref:
        return x, x_ref
    else:
        return x
