import os
import torch
import torch.nn as nn
import torch.nn.functional as F
import pandas as pd
import numpy as np
from tqdm import tqdm
import datetime
import json

import sys
sys.path.append("")

from src.models.nn_potential import ApproxPotential
from src.models.sbm import ScoreBaseModel
from src.objective.obj import objective_like_dpo
from src.potentials.potential_for_inner_loop import Potential_for_inner_loop
from src.utils.set_seed import set_seed

import matplotlib.pyplot as plt

import math

import gc

def main():
    print("## testing objective ##")
    set_seed(223)
    device = "cuda"
    model_sbm = ScoreBaseModel(2).to(device)
    model_sbm.load_state_dict(torch.load('outputs/pretrain/score_base_model.pth'))
    # paralleilze
    model_sbm = nn.DataParallel(model_sbm)
    objective_class = objective_like_dpo(device)

    import wandb
    wandb.init(project="MoG-upperbound")

    for idx_loop in range(101, 201):
        print("idx_loop: ", idx_loop)
        Potential = ApproxPotential(2).to(device)
        potential_path = "outputs/figures/dual_averaging/20240916_105414/model_potential_"+str(idx_loop)+".pth"
        Potential.load_state_dict(torch.load(potential_path))
        Potential = nn.DataParallel(Potential)
        Potential.eval()    
        upperbound = objective_class._calc_upperbound(model_sbm, model_potential=Potential)
        print("upperbound: ", upperbound)
        wandb.log({"idx_loop": idx_loop, "upperbound": upperbound})
        del Potential
        gc.collect()
        torch.cuda.empty_cache()
    
    print("## end testing objective ##")

if __name__ == "__main__":
    main()